@@ -1,19 +1,23 @@
-# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 import mforms
 
@@ -341,7 +345,7 @@ class ExplainNode(VBoxFigure):
         if text:
             if type(text) is unicode:
                 text = text.encode("utf8")
-            self._context.tooltip = mforms.newPopover(mforms.PopoverStyleTooltip)
+            self._context.tooltip = mforms.newPopover(None, mforms.PopoverStyleTooltip)
 
             if fig.is_container:
                 xx, yy = self._context.client_to_screen(x + 10, y)
@@ -1429,7 +1433,7 @@ class ExplainContext:
         if 'rows_examined_per_scan' not in table:
             table['rows_examined_per_scan'] = (table.get('rows') or 0)
         if 'rows_produced_per_join' not in table:
-            table['rows_produced_per_join'] = (table.get('rows') or 0) * int(table.get('filtered', 0)) / 100
+            table['rows_produced_per_join'] = (table.get('rows') or 0) * int(float(table.get('filtered', 0))) / 100
 
         # table nodes that have a materialized_from_subquery node are not real tables
         # so show them as a container for the subquery that it executes
