@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
@@ -48,9 +48,10 @@ GET_ACCOUNT_MYSQL_TABLE_PRIVS_QUERY = u"SELECT * FROM mysql.tables_priv WHERE Ho
 #GET_ACCOUNT_IS_TABLE_PRIVS_QUERY = "SELECT * FROM mysql.tables_priv WHERE Host='%(host)s' AND User='%(user)s' AND Db='information_schema'"
 
 CREATE_USER_QUERY = u"CREATE USER '%(user)s'@'%(host)s' IDENTIFIED BY '%(password)s'"
-CREATE_USER_QUERY_PLUGIN_AUTH_STRING = u"CREATE USER '%(user)s' IDENTIFIED WITH '%(auth_plugin)s' AS '%(auth_string)s'"
-CREATE_USER_QUERY_PLUGIN = u"CREATE USER '%(user)s' IDENTIFIED WITH '%(auth_plugin)s'"
-CREATE_USER_QUERY_PLUGIN_AUTH_CACHING = u"CREATE USER '%(user)s' IDENTIFIED WITH '%(auth_plugin)s' BY '%(password)s'"
+CREATE_USER_QUERY_PLUGIN_AUTH_STRING = u"CREATE USER '%(user)s'@'%(host)s' IDENTIFIED WITH '%(auth_plugin)s' AS '%(auth_string)s'"
+CREATE_USER_QUERY_PLUGIN = u"CREATE USER '%(user)s'@'%(host)s' IDENTIFIED WITH '%(auth_plugin)s'"
+CREATE_USER_QUERY_PLUGIN_AUTH_CACHING = u"CREATE USER '%(user)s'@'%(host)s' IDENTIFIED WITH '%(auth_plugin)s' BY '%(password)s'"
+CREATE_USER_QUERY_PLUGIN_AUTH_NATIVE = u"CREATE USER '%(user)s'@'%(host)s' IDENTIFIED WITH 'mysql_native_password' BY '%(password)s'"
 
 ALTER_USER_RESOURCES = u"ALTER USER '%(user)s'@'%(host)s'" # A WITH clause will be added
 GRANT_GLOBAL_PRIVILEGES_QUERY = u"GRANT %(granted_privs)s ON *.* TO '%(user)s'@'%(host)s'"  # A WITH clause will be added if needed
@@ -329,7 +330,7 @@ class AdminSecurity(object):
             while result.nextRow():
                 user = to_unicode(result.stringByName("User"))
                 host = to_unicode(result.stringByName("Host"))
-                accounts.append((user, host))
+                accounts.append((user, host, True))
 
 
         # Get a list of invalid privileges
@@ -356,6 +357,7 @@ class AdminSecurity(object):
         privs += get_zombies(ZOMBIE_PROCS_PRIVS_QUERY, ["Db", "Routine_name"])
         zombies = {}
         for account, priv in privs:
+            account = (account[0], account[1], True)
             if account not in zombies:
                 zombies[account] = []
             zombies[account].append(priv)
@@ -401,9 +403,17 @@ class AdminSecurity(object):
     def delete_account(self, account):
         if account.is_commited:
             self.do_delete_account(account.username, account.host)
-        del self._account_info_cache[account.username+"@"+account.host]
-        if (account.username, account.host) in self._accounts:
-          self._accounts.remove((account.username, account.host))
+            
+        if self._account_info_cache.has_key((account.username, account.host, True)):
+            del self._account_info_cache[account.username+"@"+account.host]
+            
+        if self.is_zombie(account.username, account.host):
+            del self._zombie_privs[(account.username, account.host, True)]
+    
+        if (account.username, account.host, True) in self._accounts:
+            self._accounts.remove((account.username, account.host, True))
+            
+    
 
     def revert_account(self, account, backup):
         try:
@@ -424,31 +434,33 @@ class AdminSecurity(object):
         copy.is_commited = False
         copy.username += '_copy'
         self._account_info_cache[copy.username+"@"+copy.host] = copy
-        self._accounts.append((copy.username, copy.host))
+        self._accounts.append((copy.username, copy.host, False))
         return copy
 
 
     def create_account(self):
         def unique_name(user, host, counter=None):
             name = user + ( str(counter) if counter else '' )
-            if (name, host) in self._accounts:
-                name = unique_name(user, host, counter+1 if isinstance(counter, int) else 1)
+            for (n, h, loaded) in self._accounts:
+                if name == n and host == h:
+                    name = unique_name(user, host, counter+1 if isinstance(counter, int) else 1)
+                    break
             return name
 
         acct = AdminAccount(self)
         acct.host = u"%"
         acct.username = to_unicode(unique_name(u'newuser', acct.host))
         self._account_info_cache[acct.username+"@"+acct.host] = acct
-        self._accounts.append((acct.username, acct.host))
+        self._accounts.append((acct.username, acct.host, False))
         return acct
 
 
     def is_zombie(self, user, host):
-        return self._zombie_privs.has_key((user,host))
+        return self._zombie_privs.has_key((user,host, True))
 
 
     def get_zombie_privs(self, user, host):
-        return self._zombie_privs.get((user,host), None)
+        return self._zombie_privs.get((user,host, True), None)
 
 
     def async_get_account(self, callback, name, host):
@@ -762,7 +774,7 @@ class AdminAccount(object):
             if (self.username, self.host) in self._owner.account_names:
                 raise WBSecurityValidationError("The '%s' account already exists and cannot be saved." % (self.formatted_name()))
         elif not self.is_commited:
-            if (self.username, self.host) in self._owner.account_names:
+            if (self.username, self.host, True) in self._owner.account_names:
                 raise WBSecurityValidationError("The '%s' account already exists and cannot be saved." % (self.formatted_name()))
 
         fields = {
@@ -777,9 +789,14 @@ class AdminAccount(object):
   
         password_already_set = False
         if not self.is_commited:  # This is a new account
-            if self.auth_plugin and self.auth_plugin != 'mysql_native_password':
+            if self.auth_plugin:
                 if self.auth_string is None:
                     create_query = CREATE_USER_QUERY_PLUGIN
+                elif self.auth_plugin == 'mysql_native_password':
+                    if (self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version < Version(5, 7, 0)):
+                        create_query = CREATE_USER_QUERY_PLUGIN
+                    else:
+                        create_query = CREATE_USER_QUERY_PLUGIN_AUTH_NATIVE
                 elif self.auth_plugin == 'caching_sha2_password':
                     create_query = CREATE_USER_QUERY_PLUGIN_AUTH_CACHING
                 else:
